
*     ------------------------------------------------------------------
*     program epdflib2
*     ------------------------------------------------------------------
*
*     Read spline parameterization of pdfs or stfs and
*     perform error propagation. See the epdflib manual
*     (NIKHEF-99-034) for instructions how to use the program.
*
*     This program, input files, example jobs and writeup can be
*     downloaded from http://www.nikhef.nl/user/h24/qcdnum.
*
*     If you use epdflib, please refer to the QCD analysis paper:
*
*     M. Botje, 'A QCD analysis of HERA and fixed target structure
*     function data', NIKHEF 99-011, DESY 99-038, hep-ph/9912439
*     (to appear in Eur. Phys. J. C).
*
*     Author: M. Botje, h24@nikhef.nl
*
*     ------------------------------------------------------------------
*     Date       Version  Comment
*     ------------------------------------------------------------------
*     19-08-99   01.00  - Initial version.
*     24-05-00   02.00  - Factor 15 faster than version 1.
*                       - Uses bspline representation ---> needs .pdfnt2
*                         and .stfnt2 input files.
*                       - Version check on input file.
*                       - Gives also alpha_s(Q2).
*     ------------------------------------------------------------------


C     ================================
      subroutine ep_stop(jstop,jprint)
C     ================================

C--   Set stop and print flag

      implicit double precision (a-h,o-z)

      common /epstop/ istop,iprint,iflag

      istop  = jstop
      iprint = jprint
      iflag  = 123456

      return
      end

C     ==================================
      subroutine ep_open(lun,fname,ierr)
C     ==================================

C--   Open ntuple file and make event directory
C--
C--   Input:  lun     = logical unit number
C--           fname   = Input file name
C--
C--   Output: ierr    = 0 -- OK
C--                     1 -- Cannot open/read input file
C--                     2 -- Not enough memory

      implicit double precision (a-h,o-z)

      include 'epdflib2.inc'
     
      real   par
      common /ntpbuf/ np,ip,id,nx,nq,par(mtab*mtab)

      common /epindx/ ipi(mpdf),idi(mpdf),nevent,icv(-8:-1)
      common /epchi2/ eav(mpdf),eps(mpdf)
      common /epstop/ istop,iprint,iflag
      common /parsxq/ xqpar(mtab*mtab*mpdf),npx,npq,npt
      common /epepsi/ eavg(mset),emax(mset),npts(mset)
      common /epderi/ nders(mset),idfpar(mset,mder)
      common /eppnts/ ipoint(mset,0:mder),nset
      common /eptmat/ tmat(8,-5:5)

      character*(*) fname
      save          store
      dimension     store(mtab*mtab)

      ierr = 0
      len  = lenocc(fname)
      if(len.eq.0) goto 500

C--   Initialize
      nevent = 0
      do i = -8,-1
        icv(i) = 0
      enddo
      if(iflag.ne.123456) then
        istop  = 1
        iprint = 0
      endif
      npx  = 0
      npq  = 0
      npt  = 0
      nset = 0
      call vzero(xqpar, 2*mtab*mtab*mpdf)
      call vzero(eavg,  2*mset)
      call vzero(emax,  2*mset)
      call vzero(npts,  mset)
      call vzero(nders, mset)
      call vzero(idfpar,mset*mder)
      call vzero(ipoint,mset*(mder+1))

C--   Open ntuple file
      call hropen(lun,'SPLINE',fname(1:len),' ',1024,istat)

      if(istat.ne.0) then
        goto 510
      elseif(iprint.ge.1) then
        ierr = 0
        write(6,'(/'' EP_OPEN: open file '',A,'' on lun'',I4)')
     +  fname(1:len),lun
      endif

C--   Read ntuple
      call hrin(100,99999,0)

      call hbname(100,' ',0,'$CLEAR')
      call hbname(100,'SPLINE',np,'$SET')

C--   Check number of ntuple entries
      call hnoent(100,nevent)
      if(nevent.lt.1.or.nevent.gt.mpdf) goto 520

C--   Initialize ranges (used for printout only)
      ipmin = 999999
      ipmax = 0
      nndef = 0
      idmin = 999999
      idmax = 0
      ismin = 999999
      ismax = 0
      ibmin = 999999
      ibmax = 0

C--   Make event directory
      nxrem = 0
      nqrem = 0
      do i = 1,nevent
        call hgnt(100,i,ierr)
C--     Version check
        if(ip.eq.-8) then
          if(par(1).lt.2.0.or.par(1).ge.3.0) goto 515
        endif
C--     Check that all splines have the same number of parameters
        if(ip.gt.0) then
          if(nxrem.eq.0.and.nqrem.eq.0) then
            nxrem = nx
            nqrem = nq
          elseif(nxrem.ne.nx.or.nqrem.ne.nq) then
            goto 530
          endif
        endif
C--     Check if enough space to hold the parameters
        if(np.gt.mtab*mtab) goto 540
        ipi(i) = ip
        idi(i) = id
C--     Count number of node point definitions
        if(ip.eq.-1) then
          nndef = nndef+1
C--       Check that there is only one splinedef on the file
          if(nndef.eq.1) then
            do j = 1,np
              store(j) = par(j)
            enddo
          elseif(nndef.gt.1) then
            do j = 1,np
              if(store(j).ne.par(j)) goto 550
            enddo
          endif
        endif
C--     Store address of special records
        if(ip.ge.-8.and.ip.le.-1) then
          icv(ip) = i
        endif
C--     Range of pdfids (for printout only)
        if(ip.gt.0) then
          ipmax = max(ipmax,ip)
          ipmin = min(ipmin,ip)
        endif
C--     Range of derivatives (for printout only)
        if(id.gt.0.and.id.lt.100) then
          idmax = max(idmax,id)
          idmin = min(idmin,id)
        endif
C--     Range of systematic fits (for printout only)
        if(id.gt.100.and.id.lt.200) then
          ismax = max(ismax,id-100)
          ismin = min(ismin,id-100)
        endif
C--     Range of error bands (for printout only)
        if(id.gt.200) then
          ibmax = max(ibmax,id-200)
          ibmin = min(ibmin,id-200)
        endif
C--     store eavg and epsi (just for printout)
        if(ip.ge.-8.and.ip.le.-1) then
          eav(i) = 0.
          eps(i) = 0.
        else
          eav(i) = par(np-1)
          eps(i) = par(np)
        endif
      enddo

C--   No version info found on this file
      if(icv(-8).eq.0) goto 515

      ipmin = min(ipmin,ipmax)
      idmin = min(idmin,idmax)
      ismin = min(ismin,ismax)
      ibmin = min(ibmin,ibmax)

      if(iprint.ge.2) then
        write(6,'(/
     +        '' Number of node point definitions ...:'',I5 /
     +        '' Pdf range ..........................:'',2I5/
     +        '' Derivative range ...................:'',2I5/
     +        '' Systematic fit range ...............:'',2I5/
     +        '' Error band range ...................:'',2I5)')
     +   nndef,ipmin,ipmax,idmin,idmax,ismin,ismax,ibmin,ibmax
        call ep_pdir(6)
      endif

C--   Read special records
C--   Read spline definition
      call ep_ndin(icv(-1))
C--   Read stat, syst covariance matrix and parameter list
      call ep_cvin(icv(-2),icv(-3),icv(-4))
C--   Read transformation matrix
      call ep_trin(icv(-5),icv(-6))
C--   Read alpha_s info
      call ep_asin(icv(-7))

      return

C--   Error messages
 500  continue
      write(6,'(/'' EP_OPEN: no input file given'',
     +           '' ---> STOP'')')
      ierr = 1
      stop  
 510  continue
      write(6,'(/'' EP_OPEN: cannot open file '',A,
     +           '' ---> STOP'')') fname(1:len)
      ierr = 1
      stop
 515  continue
      write(6,'(/'' EP_OPEN: cannot read input file '',A,
     +           '' ---> STOP'')') fname(1:len)
      ierr = 1
      stop
 520  continue
      write(6,'(/'' EP_OPEN: # pdfs ='',I5,'' outside range [1-'',
     +        I5,''] ---> STOP'')') nevent,mpdf
      write(6,'( '' -------> increase value of mpdf ='',I5)') mpdf
      ierr = 2
      stop
 530  continue
      write(6,'(/'' EP_OPEN: nx,nq ='',2I5,'' should be nx,nq ='',
     +       2I5,'' ---> STOP'')') nx,nq,nxrem,nqrem
      ierr = 3
      stop
 540  continue
      write(6,'(/'' EP_OPEN: # npar ='',I5,'' outside range [1-'',
     +        I5,''] ---> STOP'')') np,mtab*mtab
      write(6,'( '' -------> increase value of mtab ='',I5)') mtab
      ierr = 4
      stop
 550  continue
      write(6,'(/'' EP_OPEN: more than one splinedef'',
     +           '' on input file ---> STOP'')')
      write(6,'(/''    #          old          new'',
     +           ''         diff'')')
      do k = 1,np
        write(6,'(I5,3E13.5)') k,store(k),par(k),
     +                           store(k)-par(k)
      enddo
      ierr = 5
      stop

      end

C     ========================
      subroutine ep_ndin(ievt)
C     ========================

C--   Read spline definition

      implicit double precision (a-h,o-z)

      include 'epdflib2.inc'
     
      real   par
      common /ntpbuf/ np,ip,id,nx,nq,par(mtab*mtab)

      common /eplims/ x_mi,x_ma,q_mi,q_ma
      common /epstop/ istop,iprint,iflag

      dimension xnod(mtab),qnod(mtab)
      dimension iarr(10),darr(mtab*mtab)

C--   No spline definition found
      if(ievt.le.0) goto 500
     
C--   Read spline definition
      call hgnt(100,ievt,ierr)

C--   Check array limits
      if(nx.gt.10)        goto 510
      if(nq.gt.mtab*mtab) goto 520

      jp      = 0
      do i = 1,nx
        jp      = jp+1
        iarr(i) = par(jp)
      enddo
      do i = 1,nq
        jp      = jp+1
        darr(i) = par(jp)
      enddo

C--   Pass spline definition to mbspline
      call zz_sinp(iarr,10,darr,mtab*mtab)
      call zz_give(xnod,mtab,nxx,qnod,mtab,nqq)

C--   Set x-Q2 boundaries
      x_mi = exp(xnod(1))
      x_ma = 1.01D0
      q_mi = exp(qnod(1))
      q_ma = exp(qnod(nqq))

C--   Print boundaries
      if(iprint.ge.1) then
        write(6,'(/'' EP_NDIN: read node points,       ievt ='',I5
     +            )') ievt
      endif
      if(iprint.ge.2) then
        write(6,'(/
     +             ''          xmin  ='',E12.5,
     +             ''          xmax  ='',E12.5/
     +             ''         Q2min  ='',E12.5,
     +             ''         Q2max  ='',E12.5/)')
     +             x_mi,x_ma,q_mi,q_ma
        write(6,'('' '')')
        do i = 1,nxx
          write(6,'(9X,''Xnode'',I3,E12.5,I4)') i,exp(xnod(i))
        enddo
        write(6,'('' '')')
        do i = 1,nqq
          write(6,'(9X,''Qnode'',I3,E12.5,I4)') i,exp(qnod(i))
        enddo
      endif

      return

C--   Error messages
 
 500  continue
      write(6,'(/'' EP_NDIN: no spline definition found '',
     +           '' ---> STOP'')')
      stop

 510  continue
      write(6,'(/'' EP_NDIN: nx ='',I5,'' outside range [1-10]'',
     +           '' ---> STOP'')') nx
      write(6,'( '' -------> Probably an obsolete input file'')')
      stop

 520  continue
      write(6,'(/'' EP_NDIN: nq ='',I5,'' outside range [1-'',
     +        I5,''] ---> STOP'')') nq,mtab*mtab
      write(6,'( '' -------> Probably an obsolete input file'')')
      stop

      end

C     ===================================
      subroutine ep_lims(xmi,xma,qmi,qma)
C     ===================================

C--   Give QCD grid boundaries

      implicit double precision (a-h,o-z)

      common /eplims/ x_mi,x_ma,q_mi,q_ma

      xmi = x_mi
      xma = x_ma
      qmi = q_mi
      qma = q_ma

      return
      end

C     ===============================
      subroutine ep_trin(iepdf,iestf)
C     ===============================

C--   Read the transformation matrix from ntuple

C--   iepdf  = event# where pdf matrix is stored (pdf file only)
C--   iestf  = event# where stf matrix is stored (stf file only)

      implicit double precision (a-h,o-z)

      include 'epdflib2.inc'

      real   par
      common /ntpbuf/ np,ip,id,nx,nq,par(mtab*mtab)

      common /eptmat/ tmat(8,-5:5)
      common /epstop/ istop,iprint,iflag

      character*2 prnt(-5:5)
      data prnt /'bb','cb','sb','ub','db',' g',
     +           ' d',' u',' s',' c',' b'         /

C--   No transformation matrix found
      if(iepdf.le.0.and.iestf.le.0) goto 500

C--   Initialize
      call vzero(tmat,2*8*11)

C--   Read transformation matrix
      if(iepdf.ne.0) then
        call hgnt(100,iepdf,ierr)
      elseif(iestf.ne.0) then
        call hgnt(100,iestf,ierr)
      else
        return
      endif

C--   Store transformation matrix
      nn = 0
      do i = 1,nx
        do j = -5,5
          nn = nn+1
          tmat(i,j) = par(nn)
        enddo
      enddo

C--   Print
      if(iprint.ge.1) then
        if(iepdf.ne.0) then
          write(6,'(/'' EP_TRIN: read pdf transf matrix, ievt ='',
     +            I5)') iepdf
        endif
        if(iestf.ne.0) then
          write(6,'(/'' EP_TRIN: read stf transf matrix, ievt ='',
     +            I5)') iestf
        endif
      endif
      if(iprint.eq.2) then
      write(6,'(/'' pdf# ->     1     2     3     4'',
     +           ''     5     6     7     8'')')
      write(6,'( '' -------------------------------'',
     +           ''------------------------'')')
        do j = -5,5
          write(6,'(1X,A2,I5,8F6.1)') prnt(j),j,(tmat(i,j),i=1,8) 
        enddo
      endif

      return
 
 500  continue
      write(6,'(/'' EP_TRIN: no transformation matrix found '',
     +           '' ---> STOP'')')
      stop

      end

C     ========================================
      subroutine ep_cvin(iestat,iesyst,iepars)
C     ========================================

C--   Read the covariance matrices from ntuple

C--   iestat = event# where stat cvar matrix is stored
C--   iesyst = event# where syst cvar matrix is stored
C--   iepars = event# where parameter list   is stored

      implicit double precision (a-h,o-z)

      include 'epdflib2.inc'

      real   par
      common /ntpbuf/ np,ip,id,nx,nq,par(mtab*mtab)

      common /epcvar/ aa(mder),da(mder,3),cvar(mder,mder,3),
     +                work(mder,mder,3),ifit(mder),npar
      common /epstop/ istop,iprint,iflag

      dimension ievt(2)

C--   Initialize
      npar = 0
      call vzero(aa,2*mder)
      call vzero(da,2*mder*3)
      call vzero(cvar,2*mder*mder*3)
      call vzero(work,2*mder*mder*3)
      call vzero(ifit,mder)

C--   Read covariance matrices
      ievt(1) = iestat
      ievt(2) = iesyst

C--   Loop over stat,syst
      do 50 ie = 1,2
        if(ievt(ie).eq.0) goto 50
        call hgnt(100,ievt(ie),ierr)
        npar = nx
C--     Check array dimension
        if(npar.le.0.or.npar.ge.mder) goto 500

C--     Store covariance matrix
        nn = 0
        do i = 1,npar
          do j = i,npar
            nn = nn+1
            cvar(i,j,ie) = par(nn)
            cvar(j,i,ie) = par(nn)
          enddo
        enddo
  50  continue

C--   Store sum of both matrices and setup work arrary
      do i = 1,npar
        do j = 1,npar
          if(i.ne.j) then
            work(i,j,1) = 2.*cvar(i,j,1)
            work(i,j,2) = 2.*cvar(i,j,2)
          else
            work(i,j,1) = cvar(i,j,1)
            work(i,j,2) = cvar(i,j,2)
          endif
          cvar(i,j,3) = cvar(i,j,1) + cvar(i,j,2)
          work(i,j,3) = work(i,j,1) + work(i,j,2)
        enddo
      enddo

C--   Now read parameter list
      if(iepars.eq.0) return
      call hgnt(100,iepars,ierr)
      nn = 0
      do i = 1,nx
        nn = nn+1
        ifit(i) = par(nn)
      enddo
      do i = 1,nx
        nn = nn+1
        aa(i)   = par(nn)
      enddo
      do i = 1,nx
        nn = nn+1
        da(i,1) = par(nn)
      enddo
      do i = 1,nx
        nn = nn+1
        da(i,2) = par(nn)
      enddo
      do i = 1,nx
        da(i,3) = sqrt(da(i,1)*da(i,1)+da(i,2)*da(i,2))
      enddo

C--   Print
      if(iprint.ge.1) then
        write(6,'(/'' EP_CVIN: read error matrices,    ievt ='',
     +        3I5)') iestat,iesyst,iepars
      endif
      if(iprint.ge.2) then
      write(6,'(/''    #    parameter      staterr      systerr'',
     +           ''    stat+syst  fit'')')
        do i = 1,npar
          write(6,'(I5,4E13.5,I5)') i,aa(i),da(i,1),da(i,2),
     +    da(i,3),ifit(i)
        enddo
      endif

      return

C--   Error messages

 500  continue
      write(6,'(/'' EP_CVIN: npar ='',I5,'' not in range [1-'',
     +        I5,''] ---> STOP'')') npar,mder
      write(6,'( '' -------> increase value of mder ='',I5)') mder
      stop

      end

C     ========================
      subroutine ep_asin(ievt)
C     ========================

C--   Read alpha_s info

      implicit double precision (a-h,o-z)

      include 'epdflib2.inc'
     
      real   par
      common /ntpbuf/ np,ip,id,nx,nq,par(mtab*mtab)

      common /epstop/ istop,iprint,iflag
      common /epqmas/ udscbt(6)
      common /epaset/ qs0,as0,da0,iord

      dimension xnod(mtab),qnod(mtab)
      dimension iarr(10),darr(mtab*mtab)

C--   Initialize
      do i = 1,6
        udscbt(i) = 0.D0
      enddo
      qs0  = 0.D0
      as0  = 0.D0
      da0  = 0.D0
      iord = 0

C--   No alpha_s info on input file
      if(ievt.le.0) then
        if(iprint.ge.1)
     +  write(6,'(/'' EP_ASIN: no alphas info on input file'')')
        return
      endif
     
C--   Read alpha_s info
      call hgnt(100,ievt,ierr)

C--   Check array limits
      if(np.ne.7)        goto 500

C--   Set alpha_s info
      udscbt(4) = par(1)
      udscbt(5) = par(2)
      udscbt(6) = par(3)
      qs0       = par(4)
      as0       = par(5)
      da0       = par(6)
      iord      = par(7)

C--   Print as info
      if(iprint.ge.1) then
        write(6,'(/'' EP_ASIN: read alphas info,       ievt ='',I5
     +            )') ievt
      endif
      if(iprint.ge.2) then
        write(6,'(/
     +             ''   c, b, t    mass ='',3E12.5/
     +             ''   Q2, alfa, error ='',3E12.5/
     +             ''             order ='',I12/)')
     +  udscbt(4),udscbt(5),udscbt(6),qs0,as0,da0,iord
      endif

      return

C--   Error messages
 
 500  continue
      write(6,'(/'' EP_ASIN: np ='',I5,'' should be 7'',
     +           '' ---> STOP'')') np
      stop

      end

C     ==========================================
      subroutine ep_pars(p,d1,d2,d3,ii,nin,nout)
C     ==========================================

C--   Give info on fitted parameters

      implicit double precision (a-h,o-z)

      include 'epdflib2.inc'

      common /epcvar/ aa(mder),da(mder,3),cvar(mder,mder,3),
     +                work(mder,mder,3),ifit(mder),npar

      dimension p(*),d1(*),d2(*),d3(*),ii(*)

C--   Check array dimension
      if(nin.lt.npar) then
        write(6,'(/'' EP_PARS: dim ='',I5,'' too small for'',I5,
     +             '' parameters ---> STOP'')') nin,npar
        stop
      endif

      do i = 1,nin
        p(i)  = 0.D0
        d1(i) = 0.D0
        d2(i) = 0.D0
        d3(i) = 0.D0
        ii(i) = 0
      enddo

      do i = 1,npar
        p(i)  = aa(i)
        d1(i) = da(i,1)
        d2(i) = da(i,2)
        d3(i) = da(i,3)
        ii(i) = ifit(i)
      enddo

      nout = npar

      return
      end

C     =======================
      subroutine ep_pdir(lun)
C     =======================

C--   Print event directory on lun

      implicit double precision (a-h,o-z)

      include 'epdflib2.inc'
     
      common /epindx/ ipi(mpdf),idi(mpdf),nevent,icv(-8:-1)
      common /epchi2/ eav(mpdf),eps(mpdf)

      if(nevent.le.0) then
        write(lun,'(/'' EP_PDIR: no directory available'')')
      else
        write(lun,'(/''    #  pdf  der       eavg       emax'')')
        do i = 1,nevent
          write(lun,'(3I5,2F11.5)') i,ipi(i),idi(i),eav(i),eps(i)
        enddo
      endif

      return
      end

C     ====================================
      subroutine ep_read(jd,jt,jf,ww,ierr)
C     ====================================

C--   Read linear combination of pdfs

C--   Input: jd   = uder defined identifier [1-mset].
C--          jt   =     0 : read pdf.
C--               =     1 : read F2.
C--               =     2 : read FL.
C--               =     3 : read xF3.
C--               =     8 : read higher twist correction.
C--               =     9 : read D2 nucl correction.
C--               =    10 : read Fe nucl correction.
C--          jf   =     0 : read central value.
C--               = +/- 1 : read 1st syst fit up/dn etc.
C--          ww   = weight array ww(-5:5) in PDG convention.
C--
C--   Remark: For the systematic fits the differences
C--           sysfit - central_value are stored on the input
C--           file.

      implicit double precision (a-h,o-z)

      include 'epdflib2.inc'

      real   par
      common /ntpbuf/ np,ip,id,nx,nq,par(mtab*mtab)

      common /parsxq/ xqpar(mtab*mtab*mpdf),npx,npq,npt
      common /epepsi/ eavg(mset),emax(mset),npts(mset)
      common /epderi/ nders(mset),idfpar(mset,mder)
      common /eppnts/ ipoint(mset,0:mder),nset
      common /eptmat/ tmat(8,-5:5)
      common /epstop/ istop,iprint,iflag

      save kd,first

      dimension wt(mtyp)
      dimension ww(-5:5)

      dimension idarr(2)

      logical first
      data    first /.true./

      ierr = 0

C--   Initialize
      if(first) then
        npx  = 0
        npq  = 0
        npt  = 0
        kd   = 0
        nset = 0
        first = .false.
      endif

C--   Check array dimensions
      if(jd.le.0.or.jd.gt.mset) goto 510
      if(mtyp.ne.11)            goto 520

C--   Transform input weigts to those for the standard pdfs
      if(jt.ge.8) then
C--     For higher twist and emc corrections
        do i = 1,11
          wt(i) = 0.D0
        enddo
        wt(9) = 1.D0
      else
C--     For pdfs/stfs
        do i = 1,8
          wt(i) = 0.D0
          do j = -5,5
            wt(i) = wt(i) + tmat(i,j)*ww(j)
          enddo
        enddo
        do i = 9,11
          wt(i) = 0.D0
        enddo
      endif

C--   Initialize
      nders(jd) = 0

C--   Address where parameters are stored
C--   Increase kd every time ep_read is called
      kd = kd+1
C--   Check array boundaries
      if(kd.gt.mpdf) goto 530
C--   Pointer
      ipoint(jd,0) = kd
C--   Largest identifier up to now
      nset = max(nset,jd)

C--   Setup loop over systematic fits
C--   Central fit
      if(jf.eq.0) then
        nder     =  1
        idarr(1) =  0
        idarr(2) =  0
      endif
C--   Lower systematic fit
      if(jf.lt.0) then
        nder     =  2
        idarr(1) =  0
        idarr(2) = -100+jf
      endif
C--   Upper systematic fit
      if(jf.gt.0) then
        nder     =  2
        idarr(1) =  0
        idarr(2) =  100+jf
      endif
C--   Lower parameterization error band
      if(jf.eq.-8) then
        nder     =  2
        idarr(1) =  0
        idarr(2) = -201
      endif
C--   Upper parameterization error band
      if(jf.eq.8) then
        nder     =  2
        idarr(1) =  0
        idarr(2) =  201
      endif

C--   Kind of distribution requested (pdf, F2, FL, xF3, HT etc.)
      ityp = jt

C--   Get parameters of the central value

      do 50 ipdf = 1,mtyp

        if(wt(ipdf).eq.0.D0) goto 50

C--     Loop over systematic fits

        do k = 1,nder

          ider = idarr(k)

C--       Get event
          ievt = id_epdf(ityp,ipdf,ider)

C--       Not found
          if(ievt.eq.0) goto 530

C--       Read spline parameters and add them up
          call hgnt(100,ievt,ierr)
          npx = nx
          npq = nq
          npt = np-4
          i0  = (kd-1)*(np-4)
          do i = 1,np-4
            xqpar(i0+i) = xqpar(i0+i)+wt(ipdf)*par(i)
          enddo
C--       Numerical error of this distribution
          npts(jd) = npts(jd) + 1
          eavg(jd) = eavg(jd) + abs(wt(ipdf))*par(np-1)
          emax(jd) = emax(jd) + abs(wt(ipdf))*par(np)

        enddo

  50  continue

C--   Get parameters of all the derivatives

C--   Only when central value is requested

      if(jf.ne.0) goto 500

      nn        = 0

      do 60 ider = 1,mder

C--     See if ider is present for this linear combination
        ifound = 0
        do ipdf = 1,mtyp
          if(wt(ipdf).ne.0.D0) then
            ievt = id_epdf(ityp,ipdf,ider)
            if(ievt.ne.0) ifound = 1
          endif
        enddo

C--     This derivative is not found
        if(ifound.eq.0) goto 60

C--     Derivative found, store parameters at address kd
C--     Address where parameters are stored
        kd = kd+1
C--     Check array boundaries
        if(kd.gt.mpdf) goto 540
        nn = nn+1
        if(nn.gt.mder) goto 550
C--     Pointers
        ipoint(jd,nn) = kd
        idfpar(jd,nn) = ider
        nders(jd)     = nn

        do 55 ipdf = 1,mtyp

          if(wt(ipdf).eq.0.D0) goto 55

C--       Get event
          ievt = id_epdf(ityp,ipdf,ider)

C--       Not found
          if(ievt.eq.0) goto 55

C--       Read spline parameters and add them up
          call hgnt(100,ievt,ierr)
          npx = nx
          npq = nq
          npt = np-4
          i0  = (kd-1)*(np-4)
          do i = 1,np-4
            xqpar(i0+i) = xqpar(i0+i)+wt(ipdf)*par(i)
          enddo

  55    continue

  60  continue

C--   Print out the pointers
 500  continue
      if(iprint.ge.1) then
        write(6,'(/'' EP_READ: id ='',I4,''    itype ='',I5,
     +             ''    ifit ='',I5)') jd,jt,jf
      endif
      if(iprint.ge.2) then
        write(6,'(/'' Central value                    '',
     +             '' stored at'',I5)') ipoint(jd,0)
        do i = 1,nders(jd)
          write(6,'('' Derivative #'',I5,'' parameter#'',I5,
     +    '' stored at'',I5)') i,idfpar(jd,i),ipoint(jd,i)
        enddo
      endif

      return

C--   Error messages
 510  continue
      write(6,'(/'' EP_READ: id ='',I5,'' outside range [1-'',
     +       I5, ''] ---> STOP'')') jd,mset
      write(6,'( '' -------> increase value of mset ='',I5)') mset
      ierr = 1
      stop
 520  continue
      write(6,'(/'' EP_READ: mtyp ='',I5,'' .ne. 11'',
     +           '' ---> STOP'')') mtyp
      ierr = 2
      stop
 530  continue
      write(6,'(/'' EP_READ: cant find info for'',
     +           '' id,itype,ifit ='',3I5,
     +           '' ---> STOP'')') jd,jt,jf
      ierr = 3
      stop
 540  continue
      write(6,'(/'' EP_READ: # distns ='',I5,'' outside range [1-'',
     +        I5,''] ---> STOP'')') kd,mpdf
      write(6,'( '' -------> increase value of mpdf ='',I5)') mpdf
      ierr = 4
      stop
 550  continue
      write(6,'(/'' EP_READ: # derivs ='',I5,'' outside range [1-'',
     +        I5,''] ---> STOP'')') nn,mder
      write(6,'( '' -------> increase value of mder ='',I5)') mder
      ierr = 5
      stop
 560  continue
      write(6,'(/'' EP_READ: # xparms ='',I5,'' different from'',
     +           '' other distns'',I5,'' ---> STOP'')') nx,npx
      ierr = 6
      stop
 570  continue
      write(6,'(/'' EP_READ: # qparms ='',I5,'' different from'',
     +           '' other distns'',I5,'' ---> STOP'')') nq,npq
      ierr = 7
      stop
 580  continue
      write(6,'(/'' EP_READ: # bparms ='',I5,'' different from'',
     +           '' other distns'',I5,'' ---> STOP'')') np-4,npt
      ierr = 8
      stop

      end

C     ========================================
      integer function id_epdf(jtyp,jpdf,ider)
C     ========================================

C--   Search for event# of ityp,ipdf,ider

C--   jtyp    see input variable jt in ep_read
C--   jpdf    index of basis function stored on input file
C--           as defined by the transformation matrix tmat
C--   ider    = 0   central value
C--           # 0   derivative to fitpar_ider

      implicit double precision (a-h,o-z)

      include 'epdflib2.inc'

      common /epindx/ ipi(mpdf),idi(mpdf),nevent,icv(-8:-1)

      ipdf = 100*jtyp+jpdf

      id_epdf = 0
      do i = 1,nevent
        if(ipi(i).eq.ipdf.and.idi(i).eq.ider) id_epdf = i
      enddo

      return
      end

C     ==================================================
      double precision function ep_eavg(id,ep_emax,ierr)
C     ==================================================

C--   Get average and max numerical error

      implicit double precision (a-h,o-z)

      include 'epdflib2.inc'

      common /epepsi/ eavg(mset),emax(mset),npts(mset)
      common /eplims/ x_mi,x_ma,q_mi,q_ma

      character*7 srnam
      common /epname/ srnam

      srnam = 'EP_EAVG'

      ep_eavg = 0.D0
      ep_emax = 0.D0

      call ep_chek(id,0,1,0.5*(x_mi+x_ma),0.5*(q_mi+q_ma),ierr)
      if(ierr.ne.0) return

      ep_emax = emax(id)
      ep_eavg = eavg(id)/max(npts(id),1)

      return
      end

C     ===================================================
      double precision function ep_allf(id,jder,x,q,ierr)
C     ===================================================

C--   Calculate f or df/dp

      implicit double precision (a-h,o-z)

      external zz_fun2

      character*7 srnam
      common /epnam/ srnam

      srnam = 'EP_ALLF'

      ep_allf = ep_alls(zz_fun2,id,jder,x,q,ierr)

      return
      end

C     ===================================================
      double precision function ep_dlnx(id,jder,x,q,ierr)
C     ===================================================

C--   Calculate df/dlnx or d/dp[df/dlnx]

      implicit double precision (a-h,o-z)

      external zz_dxx2

      character*7 srnam
      common /epnam/ srnam

      srnam = 'EP_DLNX'

      ep_dlnx = ep_alls(zz_dxx2,id,jder,x,q,ierr)

      return
      end

C     ===================================================
      double precision function ep_dfdx(id,jder,x,q,ierr)
C     ===================================================

C--   Calculate df/dx or d/dp[df/dx]

      implicit double precision (a-h,o-z)

      external zz_dex2

      character*7 srnam
      common /epname/ srnam

      srnam = 'EP_DFDX'

      ep_dfdx = ep_alls(zz_dex2,id,jder,x,q,ierr)

      return
      end

C     ===================================================
      double precision function ep_dlnq(id,jder,x,q,ierr)
C     ===================================================

C--   Calculate df/dlnQ2 or d/dp[df/lnQ2]

      implicit double precision (a-h,o-z)

      external zz_dyy2

      character*7 srnam
      common /epname/ srnam

      srnam = 'EP_DLNQ'

      ep_dlnq = ep_alls(zz_dyy2,id,jder,x,q,ierr)

      return
      end

C     ===================================================
      double precision function ep_dfdq(id,jder,x,q,ierr)
C     ===================================================

C--   Calculate df/dQ2 or d/dp[df/dQ2]

      implicit double precision (a-h,o-z)

      external zz_dey2

      character*7 srnam
      common /epname/ srnam

      srnam = 'EP_DFDQ'

      ep_dfdq = ep_alls(zz_dey2,id,jder,x,q,ierr)

      return
      end

C     ===================================================
      double precision function ep_ilnx(id,jder,x,q,ierr)
C     ===================================================

C--   Calculate Int_xmi^x fdlnx or d/dp[Int_xmi^x fdlnx]

      implicit double precision (a-h,o-z)

      external zz_ixx2

      character*7 srnam
      common /epname/ srnam

      srnam = 'EP_ILNX'

      ep_ilnx = ep_alls(zz_ixx2,id,jder,x,q,ierr)

      return
      end

C     ===================================================
      double precision function ep_ifdx(id,jder,x,q,ierr)
C     ===================================================

C--   Calculate Int_xmi^x fdx or d/dp[Int_xmi^x fdx]

      implicit double precision (a-h,o-z)

      external zz_iex2

      character*7 srnam
      common /epname/ srnam

      srnam = 'EP_IFDX'

      ep_ifdx = ep_alls(zz_iex2,id,jder,x,q,ierr)

      return
      end

C     ===================================================
      double precision function ep_ilnq(id,jder,x,q,ierr)
C     ===================================================

C--   Calculate Int_qmi^q fdlnq or d/dp[Int_qmi^q fdlnq]

      implicit double precision (a-h,o-z)

      external zz_iyy2

      character*7 srnam
      common /epname/ srnam

      srnam = 'EP_ILNQ'

      ep_ilnq = ep_alls(zz_iyy2,id,jder,x,q,ierr)

      return
      end

C     ===================================================
      double precision function ep_ifdq(id,jder,x,q,ierr)
C     ===================================================

C--   Calculate Int_qmi^q fdq or d/dp[Int_qmi^q fdq]

      implicit double precision (a-h,o-z)

      external zz_iey2

      character*7 srnam
      common /epname/ srnam

      srnam = 'EP_IFDQ'

      ep_ifdq = ep_alls(zz_iey2,id,jder,x,q,ierr)

      return
      end

C     ========================================================
      double precision function ep_alls(func,id,jder,x,q,ierr)
C     ========================================================

C--   Calculate 2-dim spline function or derivative df/dp

C--   func    Spline function: f, df/dx, df/dlnx, etc.
C--   id      Identifier of pdf (as defined by ep_read)
C--   jder    Identifier of derivative
C--           = 0   calculate central value
C--           # 0   calculate df/dfitpar_jder
C--   x       Value of x
C--   q       Value of Q2
C--   ierr    Output error flag (see ep_chek)

      implicit double precision (a-h,o-z)

      include 'epdflib2.inc'

      common /epderi/ nders(mset),idfpar(mset,mder)

      external func

      ep_alls = 0.D0

C--   Check input
      call ep_chek(id,jder,1,x,q,ierr)
      if(ierr.ne.0) return

      if(jder.ne.0) then
C--     look for derivative
        ider = 0
        do i = 1,nders(id)
          ipar = idfpar(id,i)
          if(ipar.eq.jder) ider = i
        enddo
        if(ider.eq.0) return
      else
        ider = 0
      endif

      ep_alls = ep_gder(func,id,ider,x,q)

      return
      end

C     ================================================
      double precision function ep_ufun(ufun,x,q,ierr)
C     ================================================

C--   Calculate value of user function

      implicit double precision (a-h,o-z)

      external ufun   

      character*7 srnam
      common /epname/ srnam

      srnam = 'EP_UFUN'

C--   Check input
      ep_ufun = 0.D0
      call ep_chek(1,0,1,x,q,ierr)
      if(ierr.ne.0) return

      ep_ufun = ufun(0,x,q,ierr)

      return
      end

C     ====================================================
      double precision function ep_uerr(ufun,x,q,iee,ierr)
C     ====================================================

C--   Calculate error on user function

      implicit double precision (a-h,o-z)

      include 'epdflib2.inc'

      common /epcvar/ aa(mder),da(mder,3),cvar(mder,mder,3),
     +                work(mder,mder,3),ifit(mder),npar

      external ufun   

      character*7 srnam
      common /epname/ srnam

      srnam = 'EP_UERR'

C--   Check input
      ep_uerr = 0.D0
      call ep_chek(1,0,iee,x,q,ierr)
      if(ierr.ne.0) return

      val = 0.D0
      do 50 i = 1,npar
        if(ifit(i).eq.0) goto 50
        deri = ufun(i,x,q,ierr)
        do 30 j = i,npar
          if(ifit(j).eq.0) goto 30
          derj = ufun(j,x,q,ierr)
          val  = val + deri*derj*work(i,j,iee)
  30    continue
  50  continue

      ep_uerr = sqrt(val)

      return
      end

C     =======================================================
      double precision function  ep_uvar(fun1,x1,q1,
     +                                   fun2,x2,q2,iee,ierr)
C     =======================================================

C--   Calculate covar matrix element of user fun1 and fun2

      implicit double precision (a-h,o-z)

      include 'epdflib2.inc'

      common /epcvar/ aa(mder),da(mder,3),cvar(mder,mder,3),
     +                work(mder,mder,3),ifit(mder),npar

      external fun1,fun2

      character*7 srnam
      common /epname/ srnam

      srnam = 'EP_UVAR'

C--   Check input
      ep_uvar = 0.D0
      call ep_chek(1,0,iee,x1,q1,ierr)
      if(ierr.ne.0) return
      call ep_chek(1,0,iee,x2,q2,ierr)
      if(ierr.ne.0) return

      val = 0.D0
      do 50 i = 1,npar
        if(ifit(i).eq.0) goto 50
        deri = fun1(i,x1,q1,ierr)
        do 30 j = 1,npar
          if(ifit(j).eq.0) goto 30
          derj = fun2(j,x2,q2,ierr)
          val  = val + deri*derj*cvar(i,j,iee)
  30    continue
  50  continue

      ep_uvar = val

      return
      end

C     ==============================================
      double precision function ep_valu(id,x,q,ierr)
C     ==============================================

C--   Calculate value of distn id    

      implicit double precision (a-h,o-z)

      external zz_fun2

      character*7 srnam
      common /epname/ srnam

      srnam = 'EP_VALU'

      ep_valu = 0.D0

C--   Check input
      call ep_chek(id,0,1,x,q,ierr)
      if(ierr.ne.0) return

      ep_valu = ep_gder(zz_fun2,id,0,x,q)

      return
      end

C     ==================================================
      double precision function ep_errs(id,x,q,iee,ierr)
C     ==================================================

C--   Calculate error on distn id    

      implicit double precision (a-h,o-z)

      include 'epdflib2.inc'

      common /parsxq/ xqpar(mtab*mtab*mpdf),npx,npq,npt
      common /epcvar/ aa(mder),da(mder,3),cvar(mder,mder,3),
     +                work(mder,mder,3),ifit(mder),npar
      common /epderi/ nders(mset),idfpar(mset,mder)

      external zz_fun2

      character*7 srnam
      common /epname/ srnam
 
      srnam = 'EP_ERRS'

      ep_errs = 0.D0

C--   Check input
      call ep_chek(id,0,iee,x,q,ierr)
      if(ierr.ne.0) return

C--   No derivatives for this distn
      if(nders(id).eq.0) return

      val = 0.D0

      do i = 1,nders(id)
        deri = ep_gder(zz_fun2,id,i,x,q)
        ipar = idfpar(id,i)
        do j = i,nders(id)
          derj = ep_gder(zz_fun2,id,j,x,q)
          jpar = idfpar(id,j)
          val  = val + deri*derj*work(ipar,jpar,iee)
        enddo
      enddo

      ep_errs = sqrt(val)

      return
      end

C     =====================================================
      double precision function ep_cvar(id1,x1,q1,
     +                                  id2,x2,q2,iee,ierr)
C     =====================================================

C--   Calculate covar matrix element of id1 and id2

      implicit double precision (a-h,o-z)

      include 'epdflib2.inc'

      common /parsxq/ xqpar(mtab*mtab*mpdf),npx,npq,npt
      common /epcvar/ aa(mder),da(mder,3),cvar(mder,mder,3),
     +                work(mder,mder,3),ifit(mder),npar
      common /epderi/ nders(mset),idfpar(mset,mder)

      external zz_fun2

      character*7 srnam
      common /epname/ srnam

      srnam = 'EP_CVAR'

      ep_cvar = 0.D0

C--   Check input
      call ep_chek(id1,0,iee,x1,q1,ierr)
      if(ierr.ne.0) return
      call ep_chek(id2,0,iee,x2,q2,ierr)
      if(ierr.ne.0) return
      
C--   No derivatives for these distns
      if(nders(id1).eq.0) return
      if(nders(id2).eq.0) return

      val = 0.D0

      do i = 1,nders(id1)
        deri = ep_gder(zz_fun2,id1,i,x1,q1)
        ipar = idfpar(id1,i)
        do j = 1,nders(id2)
          derj = ep_gder(zz_fun2,id2,j,x2,q2)
          jpar = idfpar(id2,j)
          val  = val + deri*derj*cvar(ipar,jpar,iee)
        enddo
      enddo

      ep_cvar = val

      return
      end

C     =========================================
      subroutine ep_chek(id,ider,mode,x,q,ierr)
C     =========================================

C--   Check input

C--   ierr    -4  id   out of range
C--           -3  ider out of range
C--           -2  mode out of range
C--           -1  x or q below lower limit
C--            0  OK
C--            1  x or q above upper limit

      implicit double precision (a-h,o-z)

      include 'epdflib2.inc'

      common /eplims/ x_mi,x_ma,q_mi,q_ma
      common /eppnts/ ipoint(mset,0:mder),nset
      common /epstop/ istop,iprint,iflag

      character*7 srnam
      common /epname/ srnam

      if(id.le.0.or.id.gt.nset) then
        ierr = -4
      elseif(ider.lt.0.or.ider.gt.mder) then
        ierr = -3
      elseif(mode.lt.1.or.mode.gt.3) then
        ierr = -2
      elseif(x.lt.x_mi.or.q.lt.q_mi) then
        ierr = -1
      elseif(x.ge.x_ma.or.q.ge.q_ma) then
        ierr =  1
      else
        ierr =  0
        return
      endif

      if(istop.ne.0) then
        if(ierr.eq.-4) then
          write(6,'(/A7,'': id ='',I5,'' out of range [1-'',
     +          I5,''] ---> STOP'')') srnam,id,nset
        endif
        if(ierr.eq.-3) then
          write(6,'(/A7,'': ipder ='',I5,'' out of range [0-'',
     +          I5,''] ---> STOP'')') srnam,ider,mder
        endif
        if(ierr.eq.-2) then
          write(6,'(/A7,'': mode  ='',I5,'' out of range [1-3]'',
     +                  '' ---> STOP'')') srnam,mode
        endif
        if(ierr.eq.-1.or.ierr.eq.1) then
          write(6,'(/A7,'': x,Q2 ='',2E13.5,'' out of range '',
     +                  '' ---> STOP'')') srnam,x,q
        endif
        stop
      endif

      return
      end

C     ===================================================
      double precision function ep_gder(func,id,ider,x,q)
C     ===================================================

C--   Calculate 2-dim spline function


C--   func        Spline function: f, df/dx, df/dlnx, etc.
C--   id          Pdf indentifier
C--   ider        0 = take central value, #0 = take df/dpari
C--   x           Value of x
C--   q           Value of Q2

C--   The calling routine should have checked that that all input
C--   parameters are in range

      implicit double precision (a-h,o-z)

      include 'epdflib2.inc'

      common /parsxq/ xqpar(mtab*mtab*mpdf),npx,npq,npt
      common /eppnts/ ipoint(mset,0:mder),nset
 
      external func

      kd  = ipoint(id,ider)

C--   id, ider does not exist
      if(kd.le.0) then
        ep_gder = 0.D0
        return
      endif

      ep_gder = 1.D0

C--   Calculate function (defined on logcale!)
      i0  = (kd-1)*npt
      ep_gder = func(i0,log(x),log(q),ierr)

      return
      end

C     ===============================================
      double precision function ep_alfa(qsu,fac,ierr)
C     ===============================================

C--   Calculate alpha_s(Q2).
C--
C--   Input:   qsu  = Q2 value.
C--            fac  = scale factor: as_input = as0 + fac * da0.
C--
C--            qs0  = Q2 where as0 given.
C--            as0  = input value for alpha_s.
C--            da0  = error on as0.
C--           iord  = order of the calculation.
C--       udscbt(i) = mass of quark (i).
C--
C--   Output:  ep_alfa = alpha_s at qsu.
C--               ierr = -3  no alpha_s info available (iord = 0);
C--                    = -2  input alpha_s .le. 0;
C--                    = -1  input qsu at or below lambda2;
C--                    =  0  all OK.
 
      implicit double precision (a-h,o-z)
 
      common /epqmas/ udscbt(6)
      common /epaset/ qs0,as0,da0,iord
      common /epstop/ istop,iprint,iflag

      if(iord.ne.1.and.iord.ne.2) then
        if(istop.ne.0) goto 500
        ierr    = -3
        ep_alfa = 0.D0
        return
      endif

      if(qsu.le.0.D0) then
        if(istop.ne.0) goto 510
        ierr    = -1
        ep_alfa = 100.D0
        return
      endif

      alf = as0 + fac*da0

      if(alf.le.0.D0) then
        if(istop.ne.0) goto 520
        ierr    = -2
        ep_alfa = 0.D0
        return
      endif

      ep_alfa = ep_a0a1(qsu,qs0,alf,iord,nff,ierr)

      if(ierr.ne.0.and.istop.ne.0) goto 510

      return

C--   Error messages

 500  continue
      write(6,'(/'' EP_ALFA: no alpha_s information available'',
     +           '' ---> STOP'')')
      stop

 510  continue
      write(6,'(/'' EP_ALFA: Q2 = '',E12.5, '' below lambda2'',
     +           '' ---> STOP'')') qsu
      stop

 520  continue
      write(6,'(/'' EP_ALFA: input alphas = '',E12.5, '' .le. zero'',
     +           '' ---> STOP'')') alf
      stop

      end


C--------------------------------------------------------------

C--   Code taken from mbspline

C     =====================================
      subroutine zz_sinp(iarr,nin,darr,min)
C     =====================================

C--   Copy spline definition to bspline common blocks.
C--
C--   Input:     iarr = integer array dimensioned nin in
C--                     the calling routine.
C--               nin = dimension of iarr.
C--              darr = double precision array dimensioned min in
C--                      the calling routine.
C--               min = dimension of darr.
C--

      implicit double precision (a-h,o-z)

      include 'epdflib2.inc'

      common /tnodes/ xxtab(0:mtab,2),nxx(2),iord(2),ndmi(2),ndma(2)
      common /bpolyn/ derspl(mord,mord,mtab,2)

      dimension iarr(*), darr(*)

C--   Initialize
      call vzero(xxtab,(mtab+1)*4)
      call vzero(nxx,2)
      call vzero(iord,2)
      call vzero(ndmi,2)
      call vzero(ndma,2)
      call vzero(derspl,mord*mord*mtab*4)

C--   Check dimension
      if(nin.lt.8) goto 500
      nxx(1)  = iarr(1)
      nxx(2)  = iarr(2)
      iord(1) = iarr(3)
      iord(2) = iarr(4)
      ndmi(1) = iarr(5)
      ndmi(2) = iarr(6)
      ndma(1) = iarr(7)
      ndma(2) = iarr(8)

C--   Number of input data words
      moutx = nxx(1) + nxx(1)*iord(1)*iord(1)
      mouty = nxx(2) + nxx(2)*iord(2)*iord(2)
      mout  = moutx  + mouty
      if(mout.gt.min) goto 510

C--   Read node points
      jj = 0
      do ixy = 1,2
        do i = 1,nxx(ixy)
          jj           = jj + 1
          xxtab(i,ixy) = darr(jj)
        enddo
      enddo

C--   Read spline parameters
      do ixy = 1,2
        do ix = 1,nxx(ixy)
          do is = 1,iord(ixy)
            do id = 1,iord(ixy)
              jj                   = jj + 1
              derspl(id,is,ix,ixy) = darr(jj)
            enddo
          enddo
        enddo
      enddo

C--   Pre-calculate integrals
      call zz_bint

      return

C--   Error messages
 500  continue
      write(6,'(/'' SP_SINP: dim iarr ='',I5,'' should be .ge. 8'',
     +           '' ---> STOP'')') nin 
      stop

 510  continue
      write(6,'(/'' SP_SINP: dim darr ='',I5,'' should be .ge.'',
     +        I5,'' ---> STOP'')') min,mout
      stop

      end

C     ===================================================
      subroutine zz_give(xarr,nxin,nxout,qarr,nqin,nqout)
C     ===================================================

C--   Copy *expanded* list of node points to local arrays.
C--
C--   Output:    xarr = list of xnodes on output.
C--              nxin = dimension of xarr in calling routine.
C--             nxout = number of xnodes.
C--              qarr = list of qnodes on output.
C--              nqin = dimension of qarr in calling routine.
C--             nqout = number of qnodes.
C--

      implicit double precision (a-h,o-z)

      include 'epdflib2.inc'

      common /tnodes/ xxtab(0:mtab,2),nxx(2),iord(2),ndmi(2),ndma(2)
      common /bpolyn/ derspl(mord,mord,mtab,2)

      dimension xarr(*),qarr(*) 

C--   Check dimension
      if(nxin.lt.nxx(1)) goto 500
      if(nqin.lt.nxx(2)) goto 510

      nxout = nxx(1)
      do i = 1,nxx(1)
        xarr(i) = xxtab(i,1)
      enddo

      nqout = nxx(2)
      do i = 1,nxx(2)
        qarr(i) = xxtab(i,2)
      enddo

      return

 500  continue
      write(6,'(/'' SP_GIVE: dim xarr ='',I5,'' should be .ge.'',
     +        I5,'' ---> STOP'')') nxin,nxx(1)
      stop

 510  continue
      write(6,'(/'' SP_GIVE: dim qarr ='',I5,'' should be .ge.'',
     +        I5,'' ---> STOP'')') nqin,nxx(2)
      stop

      end

C     ============================
      subroutine zz_bspl(ixy,ix,x)
C     ============================

C--   Return iord normalized bsplines in array bsfun.
C--   The bsplines are computed as (local) polynomials.
C--   The coefficients of the polynomials are stored in
C--   the array derspl.
C--
C--   Input:          ixy  = 1 (2) for splines in x (y).
C--                    ix  = node   # as given by iz_ifrx.
C--                     x  = value of x.
C--
C--   derspl(id,is,ix,ixy) = value of bspline + derivs/n!.
C--                    id  = 1,2,...,iord: value, 1st der etc.
C--                    is  = spline # = 1,2,...,iord.
C--                   ixy  = 1 (2) splines in x (y).
C--
C--   Output:     bsfun(i) = set of normalized bsplines, i = 1,..,iord.
C--                  idmi  = index of bsfun(1).
C--                  idma  = index of bsfun(iord).
C--
C--   Remark: The calling routine should check that x and ix are
C--           in range.

      implicit double precision (a-h,o-z)

      include 'epdflib2.inc'

      common /tnodes/ xxtab(0:mtab,2),nxx(2),iord(2),ndmi(2),ndma(2)
      common /bpolyn/ derspl(mord,mord,mtab,2)
      common /gsplin/ bsfun(mord,2),idmi(2),idma(2)

C--   Calculate bsplines
      del = x-xxtab(ix,ixy)
C--   Loop over the iord non-zero bsplines for this x
      do is = 1,iord(ixy)
C--     Use Horners scheme to calculate polynomial expansion
        spl = derspl(iord(ixy),is,ix,ixy)
        do id = iord(ixy)-1,1,-1
          spl = del*spl + derspl(id,is,ix,ixy)
        enddo
        bsfun(is,ixy) = spl
      enddo

C--   Store lower/upper bspline index
      idmi(ixy) = ix-ndmi(ixy)+1
      idma(ixy) = ix-ndmi(ixy)+iord(ixy)

      return
      end

C     ============================
      subroutine zz_bdxx(ixy,ix,x)
C     ============================

C--   Return dbspline(x)/dx in array bsfun.

      implicit double precision (a-h,o-z)

      include 'epdflib2.inc'

      common /tnodes/ xxtab(0:mtab,2),nxx(2),iord(2),ndmi(2),ndma(2)
      common /bpolyn/ derspl(mord,mord,mtab,2)
      common /gsplin/ bsfun(mord,2),idmi(2),idma(2)

C--   Calculate dbspline/dx
      del = x-xxtab(ix,ixy)
C--   Loop over the iord non-zero bsplines for this x
      do is = 1,iord(ixy)
C--     Use Horners scheme to calculate polynomial expansion
        spl = derspl(iord(ixy),is,ix,ixy)*(iord(ixy)-1)
        do id = iord(ixy)-2,1,-1
          spl = del*spl + derspl(id+1,is,ix,ixy)*id
        enddo
        bsfun(is,ixy) = spl
      enddo

C--   Store lower/upper bspline index
      idmi(ixy) = ix-ndmi(ixy)+1
      idma(ixy) = ix-ndmi(ixy)+iord(ixy)

      return
      end

C     ============================
      subroutine zz_bixx(ixy,ix,x)
C     ============================

C--   Return Int_x(ix)^x bspline(z)dz in array bsfun.

      implicit double precision (a-h,o-z)

      include 'epdflib2.inc'

      common /tnodes/ xxtab(0:mtab,2),nxx(2),iord(2),ndmi(2),ndma(2)
      common /bpolyn/ derspl(mord,mord,mtab,2)
      common /gsplin/ bsfun(mord,2),idmi(2),idma(2)

C--   Calculate Int bspline dx
      del = x-xxtab(ix,ixy)
C--   Loop over the iord non-zero bsplines for this x
      do is = 1,iord(ixy)
C--     Use Horners scheme to calculate polynomial expansion
        spl = derspl(iord(ixy),is,ix,ixy)/iord(ixy)
        do id = iord(ixy)-1,1,-1
          spl = del*spl + derspl(id,is,ix,ixy)/id
        enddo
        bsfun(is,ixy) = spl*del
      enddo

C--   Store lower/upper bspline index
      idmi(ixy) = ix-ndmi(ixy)+1
      idma(ixy) = ix-ndmi(ixy)+iord(ixy)

      return
      end

C     ============================
      subroutine zz_biex(ixy,ix,x)
C     ============================

C--   Return Int_x(ix)^x bspline(z)dez in array bsfun.

      implicit double precision (a-h,o-z)

      include 'epdflib2.inc'

      common /tnodes/ xxtab(0:mtab,2),nxx(2),iord(2),ndmi(2),ndma(2)
      common /bpolyn/ derspl(mord,mord,mtab,2)
      common /gsplin/ bsfun(mord,2),idmi(2),idma(2)

C--   Calculate Int bspline dex
      x0  = xxtab(ix,ixy)
      del = x-x0
C--   Loop over the iord non-zero bsplines for this x
      do is = 1,iord(ixy)
        spl = 0.D0
C--     Loop over the terms of the taylor expansion
        do id = 1,iord(ixy)
          spl = spl + derspl(id,is,ix,ixy)*zz_znezdz(del,id-1)
        enddo
        bsfun(is,ixy) = spl*exp(x0)
      enddo

C--   Store lower/upper bspline index
      idmi(ixy) = ix-ndmi(ixy)+1
      idma(ixy) = ix-ndmi(ixy)+iord(ixy)

      return
      end

C     ========================================
      double precision function zz_znezdz(x,n)
C     ========================================

C--   Int_0^x z**n exp(z) dz

      implicit double precision (a-h,o-z)

C--   Check x.gt.0 and n.ge.0
      if(x.le.0.D0.or.n.lt.0) then
        zz_znezdz = 0.D0
        return
      endif

      dsum = x**n
      term = dsum
      xinv = 1.D0/x

      do i = n-1,0,-1
        term = -(i+1)*term*xinv
        dsum = dsum + term
      enddo

      zz_znezdz = exp(x)*dsum - term

      return
      end

C     ==================
      Subroutine zz_bint
C     ==================

C--   Calculate Int_xmin^x bspline(z)dz and store the result.
C--   Called by zz_sinp

C--   Output in bsint(is,ix,ie,ixy) = Int_x(1)^x(ix+1) bspline(z)dz.
C--                   is = spline index number.
C--                   ix = node point number.
C--                   ie = 1 ---> store Int f(z)dz.
C--                        2 ---> store Int f(z)dez.
C--                  ixy = 1 (2) spline in x (y).

      implicit double precision (a-h,o-z)

      include 'epdflib2.inc'

      common /tnodes/ xxtab(0:mtab,2),nxx(2),iord(2),ndmi(2),ndma(2)
      common /binteg/ bsint(mtab,mtab,2,2)
      common /gsplin/ bsfun(mord,2),idmi(2),idma(2)

C--   Initialization
      call vzero(bsint,2*mtab*mtab*4)

C--   Loop over x and y
      do ixy = 1,2

C--     Check
        if(ndmi(ixy).le.1)        goto 500
        if(ndma(ixy).ge.nxx(ixy)) goto 510

C--     Loop over x bins to calculate Int f(z)dz
        do ix = ndmi(ixy),ndma(ixy)
C--       Calculate all integrals for this ix
          xx = xxtab(ix+1,ixy)
          call zz_bixx(ixy,ix,xx)
C--       Store Int_x(1)^x(ix)
          do is = 1,idma(ixy)
            bsint(is,ix,1,ixy) = bsint(is,ix-1,1,ixy)
          enddo
C--       Add Int_x(ix)^x(ix+1)
          do is = idmi(ixy),idma(ixy)
            js = is-idmi(ixy)+1
            bsint(is,ix,1,ixy) = bsint(is,ix-1,1,ixy) +
     +                           bsfun(js,ixy)
          enddo
        enddo

C--     Loop over x bins to calculate Int f(z)dez
        do ix = ndmi(ixy),ndma(ixy)
C--       Calculate all integrals for this ix
          xx = xxtab(ix+1,ixy)
          call zz_biex(ixy,ix,xx)
C--       Store Int_x(1)^x(ix)
          do is = 1,idma(ixy)
            js = is-idmi(ixy)+1
            bsint(is,ix,2,ixy) = bsint(is,ix-1,2,ixy)
          enddo
C--       Add Int_x(ix)^x(ix+1)
          do is = idmi(ixy),idma(ixy)
            js = is-idmi(ixy)+1
            bsint(is,ix,2,ixy) = bsint(is,ix-1,2,ixy) +
     +                           bsfun(js,ixy)
          enddo
        enddo

      enddo

      return

C--   Error messages

 500  continue
      write(6,'(/'' ZZ_BINT: ixy ='',I5,'' ndmi = '',I5,
     +           '' should be .gt. 1 ---> STOP'')') ixy,ndmi(ixy)
      stop

 510  continue
      write(6,'(/'' ZZ_BINT: ixy ='',I5,'' ndma = '',I5,
     +           '' should be .lt.'',I5,'' ---> STOP'')') 
     +              ixy,ndma(ixy),nxx(ixy)
      stop

      end

C     ==========================
      subroutine zz_bsum(i0,sum)
C     ==========================

C--   Calculate linear combination of basis splines.
C--   Basis spine functions are stored in array bsfun

      implicit double precision (a-h,o-z)

      include 'epdflib2.inc'

      common /parsxq/ xqpar(mtab*mtab*mpdf),npx,npq,npt
      common /gsplin/ bsfun(mord,2),idmi(2),idma(2)

      sum = 0.D0
      do iy = idmi(2),idma(2)
        jy = iy-idmi(2)+1
        by = bsfun(jy,2)
        do ix = idmi(1),idma(1)
          jx  = ix-idmi(1)+1
          ii  = i0+(iy-1)*npx+ix
          sum = sum + xqpar(ii)*bsfun(jx,1)*by
        enddo
      enddo

      return
      end

C     ==============================================
      double precision function zz_fun2(i0,x,y,ierr)
C     ==============================================

C--   Calculate spline function in two dimensions.
C--
C--   Input:           i0  = base adress in array xqpar.
C--                   x,y  = value of x and y.
C--             parms(i,j) = coefficient for bsplinexi and bsplineyj.
C--
C--   Output:     zz_fun2  = value of the spline(x,y).
C--                  ierr  = 0  OK.
C--                        = 1  x,y outside range (zz_fun2 set to 0).

      implicit double precision (a-h,o-z)

      zz_fun2 = 0.D0
      ierr    = 1

C--   Find x and y bins
      ix = iz_ifrx(1,x)
      if(ix.eq.0) return
      iy = iz_ifrx(2,y)
      if(iy.eq.0) return

C--   Calculate all non-zero basis splines for this x and y
      call zz_bspl(1,ix,x)
      call zz_bspl(2,iy,y)

C--   Take linear combination of basis splines
      call zz_bsum(i0,zz_fun2)

      ierr = 0

      return
      end

C     ==============================================
      double precision function zz_dxx2(i0,x,y,ierr)
C     ==============================================

C--   Calculate dspline(x,y)/dx.

      implicit double precision (a-h,o-z)

      zz_dxx2 = 0.D0
      ierr    = 1

C--   Find x and y bins
      ix = iz_ifrx(1,x)
      if(ix.eq.0) return
      iy = iz_ifrx(2,y)
      if(iy.eq.0) return

C--   Calculate all dbspline/dx for this x
      call zz_bdxx(1,ix,x)
C--   Calculate all bsplines for this y
      call zz_bspl(2,iy,y)

C--   Take linear combination of basis splines
      call zz_bsum(i0,zz_dxx2)

      ierr = 0

      return
      end

C     ==============================================
      double precision function zz_dex2(i0,x,y,ierr)
C     ==============================================

C--   Calculate dspline(x,y)/dex = exp(-x)*dspline(x,y)/dx.

      implicit double precision (a-h,o-z)

      zz_dex2 = exp(-x)*zz_dxx2(i0,x,y,ierr)

      return
      end

C     ==============================================
      double precision function zz_dyy2(i0,x,y,ierr)
C     ==============================================

C--   Calculate dspline(x,y)/dy.

      implicit double precision (a-h,o-z)

      zz_dyy2 = 0.D0
      ierr    = 1

C--   Find x and y bins
      ix = iz_ifrx(1,x)
      if(ix.eq.0) return
      iy = iz_ifrx(2,y)
      if(iy.eq.0) return

C--   Calculate all bsplines for this x
      call zz_bspl(1,ix,x)
C--   Calculate all dbspline/dy for this y
      call zz_bdxx(2,iy,y)

C--   Take linear combination of basis splines
      call zz_bsum(i0,zz_dyy2)

      ierr = 0

      return
      end

C     ==============================================
      double precision function zz_dey2(i0,x,y,ierr)
C     ==============================================

C--   Calculate dspline(x,y)/dey = exp(-y)*dspline(x,y)/dy.

      implicit double precision (a-h,o-z)

      zz_dey2 = exp(-y)*zz_dyy2(i0,x,y,ierr)

      return
      end

C     ==============================================
      double precision function zz_ixx2(i0,x,y,ierr)
C     ==============================================

C--   Calculate Int_xmin^x spline(z,y)dz

      implicit double precision (a-h,o-z)

      include 'epdflib2.inc'

      common /tnodes/ xxtab(0:mtab,2),nxx(2),iord(2),ndmi(2),ndma(2)
      common /parsxq/ xqpar(mtab*mtab*mpdf),npx,npq,npt
      common /gsplin/ bsfun(mord,2),idmi(2),idma(2)
      common /binteg/ bsint(mtab,mtab,2,2)

      zz_ixx2 = 0.D0
      ierr    = 1

C--   Find x and y bins
      ix = iz_ifrx(1,x)
      if(ix.eq.0) return
      iy = iz_ifrx(2,y)
      if(iy.eq.0) return

C--   Calculate all bsplines for this y
      call zz_bspl(2,iy,y)
C--   Calculate all integrals from x(ix) to x
      call zz_bixx(1,ix,x)

C--   Linear combination of all integrals upto x(ix)
      do iy = idmi(2),idma(2)
        jy = iy-idmi(2)+1
        by = bsfun(jy,2)
        do is = 1,idma(1)
          ii  = i0+(iy-1)*npx+is
          zz_ixx2 = zz_ixx2 + xqpar(ii)*bsint(is,ix-1,1,1)*by
        enddo
      enddo

C--   Now add integrals from x(ix) to x
      call zz_bsum(i0,sum)
      zz_ixx2 = zz_ixx2+sum

      ierr = 0

      return
      end

C     ==============================================
      double precision function zz_iex2(i0,x,y,ierr)
C     ==============================================

C--   Calculate Int_xmin^x spline(z,y)dz

      implicit double precision (a-h,o-z)

      include 'epdflib2.inc'

      common /tnodes/ xxtab(0:mtab,2),nxx(2),iord(2),ndmi(2),ndma(2)
      common /parsxq/ xqpar(mtab*mtab*mpdf),npx,npq,npt
      common /gsplin/ bsfun(mord,2),idmi(2),idma(2)
      common /binteg/ bsint(mtab,mtab,2,2)

      zz_iex2 = 0.D0
      ierr    = 1

C--   Find x and y bins
      ix = iz_ifrx(1,x)
      if(ix.eq.0) return
      iy = iz_ifrx(2,y)
      if(iy.eq.0) return

C--   Calculate all bsplines for this y
      call zz_bspl(2,iy,y)
C--   Calculate all integrals from x(ix) to x
      call zz_biex(1,ix,x)

C--   Linear combination of all integrals upto x(ix)
      do iy = idmi(2),idma(2)
        jy = iy-idmi(2)+1
        by = bsfun(jy,2)
        do is = 1,idma(1)
          ii  = i0+(iy-1)*npx+is
          zz_iex2 = zz_iex2 + xqpar(ii)*bsint(is,ix-1,2,1)*by
        enddo
      enddo

C--   Now add integrals from x(ix) to x
      call zz_bsum(i0,sum)
      zz_iex2 = zz_iex2+sum

      ierr = 0

      return
      end

C     ==============================================
      double precision function zz_iyy2(i0,x,y,ierr)
C     ==============================================

C--   Calculate Int_ymin^y spline(x,t)dt

      implicit double precision (a-h,o-z)

      include 'epdflib2.inc'

      common /tnodes/ xxtab(0:mtab,2),nxx(2),iord(2),ndmi(2),ndma(2)
      common /parsxq/ xqpar(mtab*mtab*mpdf),npx,npq,npt
      common /gsplin/ bsfun(mord,2),idmi(2),idma(2)
      common /binteg/ bsint(mtab,mtab,2,2)

      zz_iyy2 = 0.D0
      ierr    = 1

C--   Find x and y bins
      ix = iz_ifrx(1,x)
      if(ix.eq.0) return
      iy = iz_ifrx(2,y)
      if(iy.eq.0) return

C--   Calculate all bsplines for this x
      call zz_bspl(1,ix,x)
C--   Calculate all integrals from y(iy) to y
      call zz_bixx(2,iy,y)

C--   Linear combination of all integrals upto y(iy)
      do is = 1,idma(2)
        by = bsint(is,iy-1,1,2)
        do ix = idmi(1),idma(1)
          jx = ix-idmi(1)+1
          ii = i0+(is-1)*npx+ix
          zz_iyy2 = zz_iyy2 + xqpar(ii)*bsfun(jx,1)*by
        enddo
      enddo

C--   Now add integrals from x(ix) to x
      call zz_bsum(i0,sum)
      zz_iyy2 = zz_iyy2+sum

      ierr = 0

      return
      end

C     ==============================================
      double precision function zz_iey2(i0,x,y,ierr)
C     ==============================================

C--   Calculate Int_ymin^y spline(x,t)dexp^t

      implicit double precision (a-h,o-z)

      include 'epdflib2.inc'

      common /tnodes/ xxtab(0:mtab,2),nxx(2),iord(2),ndmi(2),ndma(2)
      common /parsxq/ xqpar(mtab*mtab*mpdf),npx,npq,npt
      common /gsplin/ bsfun(mord,2),idmi(2),idma(2)
      common /binteg/ bsint(mtab,mtab,2,2)

      zz_iey2 = 0.D0
      ierr    = 1

C--   Find x and y bins
      ix = iz_ifrx(1,x)
      if(ix.eq.0) return
      iy = iz_ifrx(2,y)
      if(iy.eq.0) return

C--   Calculate all bsplines for this x
      call zz_bspl(1,ix,x)
C--   Calculate all integrals from y(iy) to y
      call zz_biex(2,iy,y)

C--   Linear combination of all integrals upto y(iy)
      do is = 1,idma(2)
        by = bsint(is,iy-1,2,2)
        do ix = idmi(1),idma(1)
          jx = ix-idmi(1)+1
          ii = i0+(is-1)*npx+ix
          zz_iey2 = zz_iey2 + xqpar(ii)*bsfun(jx,1)*by
        enddo
      enddo

C--   Now add integrals from x(ix) to x
      call zz_bsum(i0,sum)
      zz_iey2 = zz_iey2+sum

      ierr = 0

      return
      end

C     ===============================
      integer function iz_ifrx(ixy,x)
C     ===============================

C--   Find node point ix below x
C--
C--   Input:  x       = x value.
C--   Output: iz_ifrx = node point index.
C--                   = 0 if x out of range.

      implicit double precision (a-h,o-z)

      include 'epdflib2.inc'

      common /tnodes/ xxtab(0:mtab,2),nxx(2),iord(2),ndmi(2),ndma(2)

      save last
      dimension last(2)
      data last /1,1/

      il = last(ixy)
      nx = nxx(ixy)

C--   Quick check if x in same bin as last time
      if(x.ge.xxtab(il,ixy).and.x.lt.xxtab(il+1,ixy)) then
        iz_ifrx = il
C--   Check if x is in range
      elseif(x.ge.xxtab(1,ixy).and.x.lt.xxtab(nx,ixy)) then
C--     search bin
        iz_ifrx  = 0
        last(ixy) = 1
        do i = nx-1,1,-1
          if(x.ge.xxtab(i,ixy)) then
            iz_ifrx  = i
            last(ixy) = i
            return
          endif
        enddo
      else
C--     Value of x out of range
        iz_ifrx  = 0
        last(ixy) = 1
      endif

      return
      end

C--------------------------------------------------------------

C--   Code taken from qcdnum

C     ============================================================
      double precision function ep_a0a1(qsu,qs0,as0,iord,nff,ierr)
C     ============================================================
 
      implicit double precision (a-h,o-z)
 
      common /epqmas/ udscbt(6)

      qs1   = qsu

      qmu0  = sqrt(qs0)
      qmu1  = sqrt(qs1)
 
      do i=1,6
        if(qmu0.ge.udscbt(i)) nf0 = i
        if(qmu1.ge.udscbt(i)) nf1 = i
      enddo
 
      if(nf1.lt.nf0) then
        ist = -1
        jst =  0
      else
        ist = 1
        jst = 1
      endif
 
      alfa0 = as0
      q00   = qs0
 
      do nf = nf0,nf1,ist
        if(nf.ne.nf1) then
          q21 = udscbt(nf+jst)*udscbt(nf+jst)
        else
          q21 = qs1
        endif
        alfa1 = ep_alphar(q21,q00,alfa0,nf,iord,ierr)
        alfa0 = alfa1
        q00   = q21
      enddo
 
      ep_a0a1 = alfa0
 
      return
      end

C     =============================================================
      double precision function ep_alphar(qsq,qs0,as0,nf,iord,ierr)
C     =============================================================
 
      implicit double precision (a-h,o-z)
 
C--   alphas from rge given as0 at qs0
 
      data pi / 3.14159265359 /

      bet0 = 11.-2*nf/3.
      bet1 = 102.-38*nf/3.
      b0   = bet0/(4.*pi)
      b1   = bet1/(4.*pi*bet0)
      ierr = 0

      term0 = 1./as0+b0*log(qsq/qs0)
      if(term0.le.0.) then
        ep_alphar = 100.
        ierr      = -1
        return
      endif
      alfa0 = 1./term0
      if(iord.eq.1) then
        ep_alphar = alfa0
        return
      endif
  20  continue
      arg   = (1./alfa0+b1)/(1./as0+b1)
      if(arg.le.0.) then
        ep_alphar = 100.
        ierr      = -1
        return
      else 
        term  = term0+b1*log(arg)
        if(term.le.0) then
          ep_alphar = 100.
          ierr      = -1
          return
        else
          alfa1 = 1./term
        endif
      endif 
      if(abs(alfa1-alfa0).gt.1.D-6) then
        alfa0 = alfa1
        goto 20
      endif
 
      ep_alphar = alfa1
 
      return
      end
